@@ -1,7 +1,7 @@
 ;;; ess-lsp-l.el --- Support for editing Lisp source code
 
 ;; Copyright (C) 1997 A.J. Rossini.
-;; Copyright (C) 1998--2004 A.J. Rossini, Rich M. Heiberger, Martin
+;; Copyright (C) 1998--2004 A.J. Rossini, Richard M. Heiberger, Martin
 ;;      Maechler, Kurt Hornik, Rodney Sparapani, and Stephen Eglen.
 
 ;; Author: A.J. Rossini <rossini@stat.sc.edu>
@@ -20,9 +20,8 @@
 ;; MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 ;; GNU General Public License for more details.
 
-;; You should have received a copy of the GNU General Public License
-;; along with GNU Emacs; see the file COPYING.  If not, write to
-;; the Free Software Foundation, 675 Mass Ave, Cambridge, MA 02139, USA.
+;; A copy of the GNU General Public License is available at
+;; http://www.r-project.org/Licenses/
 
 ;;; Commentary:
 
@@ -48,7 +47,7 @@
     (adaptive-fill-mode           . nil)
     (indent-line-function         . 'lisp-indent-line)
     (indent-region-function       . 'lisp-indent-region)
-    (require-final-newline        . t)
+    (require-final-newline        . mode-require-final-newline)
     (comment-start                . ";")
     (comment-start-skip           . "\\(\\(^\\|[^\\\\\n]\\)\\(\\\\\\\\\\)*\\);+ *")
     (comment-column               . 40)
