@@ -33,13 +33,11 @@
 ;;; Code:
 
 (require 'cl)
+(require 'ess-generics)
+(require 'ess-utils)
 
 (autoload 'ess-turn-on-eldoc            "ess-r-d" "" nil)
-;; (autoload 'ess-ddeclient-p              "ess-inf" "(autoload)" nil)
-(autoload 'ess-dump-object-ddeclient        "ess-dde" "(autoload)" nil)
-(autoload 'SAS                              "ess-sas-d.el" "(autoload)" t)
-
-(require 'ess-utils)
+(autoload 'SAS-menu                     "ess-sas-d.el" "(autoload)" t)
 
 (defun ess-line-end-position (&optional N)
   "return the 'point' at the end of N lines. N defaults to 1, i.e., current line."
@@ -66,7 +64,7 @@
   (let ((map (make-sparse-keymap)))
 
     ;; By popular demand:
-    (define-key map "\C-m"       'newline-and-indent); = [RETURN]
+    (define-key map "\C-m"       'ess-newline-and-indent); = [RETURN]
     (define-key map [remap yank] 'ess-yank)
 
     (define-key map "\C-c\C-r"   'ess-eval-region)
@@ -130,10 +128,6 @@
   "Keymap for `ess-mode'.")
 
 ;; Redefine substituted commands
-(substitute-key-definition 'newline-and-indent
-                           'ess-newline-and-indent
-                           ess-mode-map global-map)
-
 (substitute-key-definition 'indent-new-comment-line
                            'ess-indent-new-comment-line
                            ess-mode-map global-map)
@@ -165,6 +159,8 @@
     (define-key ess-extra-map "i" 'ess-install-library)
     (define-key ess-extra-map "\C-l" 'ess-load-library)
     (define-key ess-extra-map "l" 'ess-load-library)
+    (define-key ess-extra-map "\C-r" 'inferior-ess-reload)
+    (define-key ess-extra-map "r" 'inferior-ess-reload)
     (define-key ess-extra-map "\C-s" 'ess-set-style)
     (define-key ess-extra-map "s" 'ess-set-style)
     (define-key ess-extra-map "\C-t" 'ess-build-tags-for-directory)
@@ -281,22 +277,6 @@
     ["Send bug report"  ess-submit-bug-report           t]
     ))
 
-
-
-;; (defun test-gen-menu (men)
-;;   '(
-;;     ["About editing" (ess-goto-info "Editing")  t]
-;;     ["Read ESS info" (ess-goto-info "") t]
-;;     ["Send bug report"  ess-submit-bug-report           t]))
-
-(defun SAS-menu ()
-  "Start SAS from the menu."
-  (interactive)
-  (if ess-microsoft-p
-      ;; replace with other choices for starting SAS under XEmacs?
-      (error "SAS cannot be started this way in ESS on Windows.")
-    (SAS)))
-
 (defun ess-mode-xemacs-menu ()
   "Hook to install `ess-mode' menu for XEmacs (w/ easymenu)."
   (if 'ess-mode
@@ -847,7 +827,7 @@ With prefix argument, only shows the errors ESS reported."
   "Insert character and correct line's indentation."
   (interactive "P")
   ;; skeleton-pair takes precedence
-  (if (and (boundp 'skeleton-pair) skeleton-pair (featurep 'skeleton))
+  (if (fboundp 'skeleton-pair-insert-maybe)
       (skeleton-pair-insert-maybe "{")
     ;; else
     (let (insertpos)
@@ -1008,9 +988,7 @@ generate the source buffer."
   (interactive
    (progn
      (ess-force-buffer-current "Process to dump from: ")
-     (if (ess-ddeclient-p)
-         (list (read-string "Object to edit: "))
-       (ess-read-object-name "Object to edit"))))
+     (ess-read-object-name "Object to edit")))
 
   (let* ((dirname (file-name-as-directory
                    (if (stringp ess-source-directory)
@@ -1049,49 +1027,44 @@ generate the source buffer."
           ;; No buffer and no file
           (ess-dump-object object filename))))))
 
-(defun ess-dump-object (object filename)
+(ess-defgeneric ess-dump-object (object filename)
   "Dump the ESS object OBJECT into file FILENAME."
   (let ((complete-dump-command (format inferior-ess-dump-command
                                        object filename)))
     (if (file-writable-p filename) nil
       (error "Can't dump %s as %f is not writeable." object filename))
 
-    (if (ess-ddeclient-p)
-        ;; ddeclient version
-        (ess-dump-object-ddeclient object filename)
+    (:override
+     ;; Make sure we start fresh
+     (if (get-file-buffer filename)
+         (kill-buffer (get-file-buffer filename)))
 
-      ;; else: "normal", non-DDE behavior:
+     (ess-command complete-dump-command)
+     (message "Dumped in %s" filename)
 
-      ;; Make sure we start fresh
-      (if (get-file-buffer filename)
-          (kill-buffer (get-file-buffer filename)))
+     (ess-find-dump-file-other-window filename)
 
-      (ess-command complete-dump-command)
-      (message "Dumped in %s" filename)
+     ;; PD, 1Apr97
+     ;;This ensures that the object gets indented according to ess-mode,
+     ;;not as the R/S deparser does it. At the same time, it gets rid
+     ;;of the mess generated by sending TAB characters to the readline
+     ;;functions in R when you eval-buffer-*.
+     (indent-region (point-min-marker) (point-max-marker) nil)
+     (set-buffer-modified-p nil) ; no need to safe just because of indenting
 
-      (ess-find-dump-file-other-window filename)
+     ;; Don't make backups for temporary files; it only causes clutter.
+     ;; The ESS object itself is a kind of backup, anyway.
+     (unless ess-keep-dump-files
+       (make-local-variable 'make-backup-files)
+       (setq make-backup-files nil))
 
-      ;; PD, 1Apr97
-      ;;This ensures that the object gets indented according to ess-mode,
-      ;;not as the R/S deparser does it. At the same time, it gets rid
-      ;;of the mess generated by sending TAB characters to the readline
-      ;;functions in R when you eval-buffer-*.
-      (indent-region (point-min-marker) (point-max-marker) nil)
-      (set-buffer-modified-p nil) ; no need to safe just because of indenting
+     ;; Don't get confirmation to delete dumped files when loading
+     (if (eq ess-keep-dump-files 'check)
+         (setq ess-keep-dump-files nil))
 
-      ;; Don't make backups for temporary files; it only causes clutter.
-      ;; The ESS object itself is a kind of backup, anyway.
-      (unless ess-keep-dump-files
-        (make-local-variable 'make-backup-files)
-        (setq make-backup-files nil))
-
-      ;; Don't get confirmation to delete dumped files when loading
-      (if (eq ess-keep-dump-files 'check)
-          (setq ess-keep-dump-files nil))
-
-      ;; Delete the file if necessary
-      (if ess-delete-dump-files
-          (delete-file (buffer-file-name))))))
+     ;; Delete the file if necessary
+     (if ess-delete-dump-files
+         (delete-file (buffer-file-name))))))
 
 (defun ess-find-dump-file-other-window (filename)
   "Find ESS source file FILENAME in another window."
