@@ -39,36 +39,36 @@
 If an ESS process is not associated with the buffer, do not try
 to look up any doc strings."
   (interactive)
-  (let ((proc (ess-get-next-available-process)))
-    (when proc
-     (let ((funname (or (and ess-eldoc-show-on-symbol ;; aggressive completion
-                             (thing-at-point 'symbol))
-                        (car (ess--funname.start)))))
-       (when funname
-         (let* ((args (ess-function-arguments funname proc))
-                (bargs (cadr args))
-                (doc (mapconcat (lambda (el)
-                                  (if (equal (car el) "...")
-                                      "..."
-                                    (concat (car el) "=" (cdr el))))
-                                bargs ", "))
-                (margs (nth 2 args))
-                (W (- (window-width (minibuffer-window)) (+ 4 (length funname))))
-                doc1)
-           (when doc
-             (setq doc (ess-eldoc-docstring-format funname doc))
-             (when (and margs (< (length doc1) W))
-               (setq doc1 (concat doc (propertize "  || " 'face font-lock-function-name-face)))
-               (while (and margs (< (length doc1) W))
-                 (let ((head (pop margs)))
-                   (unless (assoc head bargs)
-                     (setq doc doc1
-                           doc1 (concat doc1 head  "=, ")))))
-               (when (equal (substring doc -2) ", ")
-                 (setq doc (substring doc 0 -2)))
-               (when (and margs (< (length doc) W))
-                 (setq doc (concat doc " {--}"))))
-             doc)))))))
+  (when (and eldoc-mode ess-can-eval-in-background)
+    (let* ((proc (ess-get-next-available-process))
+           (funname (and proc (or (and ess-eldoc-show-on-symbol ;; Aggressive completion
+                                       (thing-at-point 'symbol))
+                                  (car (ess--funname.start))))))
+      (when funname
+        (let* ((args (ess-function-arguments funname proc))
+               (bargs (cadr args))
+               (doc (mapconcat (lambda (el)
+                                 (if (equal (car el) "...")
+                                     "..."
+                                   (concat (car el) "=" (cdr el))))
+                               bargs ", "))
+               (margs (nth 2 args))
+               (W (- (window-width (minibuffer-window)) (+ 4 (length funname))))
+               doc1)
+          (when doc
+            (setq doc (ess-eldoc-docstring-format funname doc))
+            (when (and margs (< (length doc1) W))
+              (setq doc1 (concat doc (propertize "  || " 'face font-lock-function-name-face)))
+              (while (and margs (< (length doc1) W))
+                (let ((head (pop margs)))
+                  (unless (assoc head bargs)
+                    (setq doc doc1
+                          doc1 (concat doc1 head  "=, ")))))
+              (when (equal (substring doc -2) ", ")
+                (setq doc (substring doc 0 -2)))
+              (when (and margs (< (length doc) W))
+                (setq doc (concat doc " {--}"))))
+            doc))))))
 
 (defun ess-eldoc-docstring-format (funname doc)
   (save-match-data
@@ -79,8 +79,7 @@ to look up any doc strings."
            ;; resize of the echo area.
            (W (1- (- (window-width (minibuffer-window))
                      (+ 2 (length funname)))))
-           newdoc
-           )
+           newdoc)
       (setq doc
             (if (or (<= (length doc) W)
                     (null ess-eldoc-abbreviation-style)
@@ -233,12 +232,12 @@ command may be necessary if you modify an attached dataframe."
         ;; always return a non-nil value to prevent history expansions
         (or (comint-dynamic-simple-complete  pattern components) 'none))))
 
-(defun ess-R-get-rcompletions (&optional start end prefix)
+(defun ess-R-get-rcompletions (&optional start end prefix allow-3-dots)
   "Call R internal completion utilities (rcomp) for possible completions.
 Optional START and END delimit the entity to complete, default to
-bol and point. If PREFIX is given, perform completion on
-PREFIX. First element of the returned list is the completion
-token. Needs version of R>=2.7.0."
+bol and point.  If PREFIX is given, perform completion on
+PREFIX.  First element of the returned list is the completion
+token.  Needs version of R >= 2.7.0."
   (let* ((start (or start
                     (if prefix
                         0
@@ -247,9 +246,13 @@ token. Needs version of R>=2.7.0."
          (prefix (or prefix (buffer-substring start end)))
          ;; (opts1 (if no-args "op<-rc.options(args=FALSE)" ""))
          ;; (opts2 (if no-args "rc.options(op)" ""))
-         (cmd (format ".ess_get_completions(\"%s\", %d)\n"
-                      (ess-quote-special-chars prefix)
-                      (- end start))))
+         (call1 (format ".ess_get_completions(\"%s\", %d)"
+                        (ess-quote-special-chars prefix)
+                        (- end start)))
+         (cmd (if allow-3-dots
+                  (concat call1 "\n")
+                (concat "local({ r <- " call1 "; r[r != '...='] })\n"))))
+
     (ess-get-words-from-vector cmd)))
 
 (defun ess-R-complete-object-name ()
