@@ -116,6 +116,8 @@
     (define-prefix-command 'ess-r-package-dev-map)
     (define-key ess-r-package-dev-map "\C-s" 'ess-r-package-set-package)
     (define-key ess-r-package-dev-map "s"    'ess-r-package-set-package)
+    (define-key ess-r-package-dev-map "\C-a" 'ess-r-devtools-ask)
+    (define-key ess-r-package-dev-map "a"    'ess-r-devtools-ask)
     (define-key ess-r-package-dev-map "\C-c" 'ess-r-devtools-check-package)
     (define-key ess-r-package-dev-map "c"    'ess-r-devtools-check-package)
     (define-key ess-r-package-dev-map "\C-d" 'ess-r-devtools-document-package)
@@ -190,16 +192,66 @@
 (easy-menu-add-item inferior-ess-mode-menu nil ess-tracebug-menu "end-dev")
 
 
-;; modify S Syntax table:
-(setq R-syntax-table S-syntax-table)
+;; Inherit from the S syntax table:
+(setq ess-r-syntax-table (copy-syntax-table S-syntax-table))
+
+;; Letting Emacs treat backquoted names and %ops% as strings solves
+;; many problems with regard to nested strings and quotes
+(modify-syntax-entry ?` "\"" ess-r-syntax-table)
+(modify-syntax-entry ?% "\"" ess-r-syntax-table)
+
+;; Underscore is valid in R symbols
+(modify-syntax-entry ?_ "_" ess-r-syntax-table)
+(modify-syntax-entry ?: "." ess-r-syntax-table)
+(modify-syntax-entry ?@ "." ess-r-syntax-table)
+(modify-syntax-entry ?$ "." ess-r-syntax-table)
+
+;; TOTHINK: Prevent string delimiting characters from messing up output in the
+;; inferior buffer
+(setq inferior-ess-r-syntax-table (copy-syntax-table ess-r-syntax-table))
+;; (modify-syntax-entry ?\' "." inferior-ess-r-syntax-table)
+;; (modify-syntax-entry ?\" "." inferior-ess-r-syntax-table)
+;; (modify-syntax-entry ?` "." inferior-ess-r-syntax-table)
+(modify-syntax-entry ?% "." inferior-ess-r-syntax-table)
+
+(defvar ess-r-completion-syntax-table
+  (let ((table (make-syntax-table ess-r-syntax-table)))
+    (modify-syntax-entry ?. "_" table)
+    (modify-syntax-entry ?: "_" table)
+    (modify-syntax-entry ?$ "_" table)
+    (modify-syntax-entry ?@ "_" table)
+    table)
+  "Syntax table used for completion and help symbol lookup.
+It makes underscores and dots word constituent chars.")
+
+(defun ess-r-font-lock-syntactic-face-function (state)
+  (let ((string-end (save-excursion
+                      (and (nth 3 state)
+                           (ess-goto-char (nth 8 state))
+                           (ess-forward-sexp)
+                           (point)))))
+    (when (eq (nth 3 state) ?`)
+      (put-text-property (nth 8 state) string-end 'ess-r-backquoted t))
+    (cond
+     ((eq (nth 3 state) ?%)
+      'ess-%op%-face)
+     ((save-excursion
+        (and (ess-goto-char string-end)
+             (ess-looking-at "<-")
+             (ess-goto-char (match-end 0))
+             (ess-looking-at "function\\b" t)))
+      font-lock-function-name-face)
+     ((save-excursion
+        (and (ess-goto-char string-end)
+             (ess-looking-at "(")))
+      ess-function-call-face)
+     ((eq (nth 3 state) ?`)
+      'ess-backquoted-face)
+     ((nth 3 state)
+      font-lock-string-face)
+     (t
+      font-lock-comment-face))))
 
-;; In R 2.x, back tick now is a quote character, so lets tell Emacs
-;; that it is; the problem below for older R should no longer be a
-;; serious issue.
-;;R >= 1.8: back tick `string` -- unfortunately no *pair* checking:
-;; breaks when things like `..' are used:
-(modify-syntax-entry ?` "\"" R-syntax-table)
-(modify-syntax-entry ?_  "_"  R-syntax-table) ; foo_bar is symbol in R >=1.9
 
 (ess-message "[ess-r-d:] (autoload ..) & (def** ..)")
 
@@ -211,7 +263,7 @@
      (ess-dialect                       . "R")
      (ess-suffix                        . "R")
      (ess-ac-sources                    . '(ac-source-R))
-     (ess-company-backends              . '((company-R-args company-R-objects :sorted)))
+     (ess-company-backends              . '((company-R-args company-R-objects :separate)))
      (ess-build-tags-command            . "rtags('%s', recursive = TRUE, pattern = '\\\\.[RrSs](rw)?$',ofile = '%s')")
      (ess-traceback-command             . "local({cat(geterrmessage(), \"---------------------------------- \n\", fill=TRUE);try(traceback(), silent=TRUE)})\n")
      (ess-call-stack-command            . "traceback(1)\n")
@@ -221,7 +273,8 @@
                                            ess-dump-filename-template-proto))
      (ess-build-help-command-function   . #'ess-r-build-help-command)
      (ess-help-web-search-command       . 'ess-R-sos)
-     (ess-mode-syntax-table             . R-syntax-table)
+     (ess-mode-syntax-table             . ess-r-syntax-table)
+     (ess-mode-completion-syntax-table      . ess-r-completion-syntax-table)
      (ess-mode-editing-alist            . R-editing-alist)
      (ess-change-sp-regexp              . ess-R-change-sp-regexp)
      (ess-help-sec-regex                . ess-help-R-sec-regex)
@@ -243,6 +296,7 @@
      ;;harmful for shell-mode's C-a: -- but "necessary" for ESS-help?
      (inferior-ess-start-file	          . nil) ;; "~/.ess-R"
      (inferior-ess-start-args           . "")
+     (inferior-ess-mode-syntax-table    . inferior-ess-r-syntax-table)
      (ess-error-regexp-alist            . ess-R-error-regexp-alist)
      (ess-describe-object-at-point-commands . 'ess-R-describe-object-at-point-commands)
      (ess-STERM                         . "iESS")
@@ -251,7 +305,8 @@
      (prettify-symbols-alist            . '(("<-" . ?←)
                                             ("<<-" . ?↞)
                                             ("->" . ?→)
-                                            ("->>" . ?↠))))
+                                            ("->>" . ?↠)))
+     (font-lock-syntactic-face-function . #'ess-r-font-lock-syntactic-face-function))
    S-common-cust-alist)
   "Variables to customize for R -- set up later than emacs initialization.")
 
@@ -264,6 +319,8 @@
             '(ess--extract-default-fl-keywords ess-R-font-lock-keywords))
     (setcdr (assoc 'ess-font-lock-keywords S-alist)
             (quote 'ess-R-font-lock-keywords))
+    (setcdr (assoc 'ess-mode-syntax-table S-alist)
+            (quote ess-r-syntax-table))
     S-alist)
   "General options for editing R source files.")
 
@@ -413,12 +470,24 @@ will be prompted to enter arguments interactively."
 Executed in process buffer."
   (interactive)
 
-  (when ess-can-eval-in-background
-    (ess-async-command-delayed
-     "invisible(installed.packages())\n" nil (get-process ess-local-process-name)
-     ;; "invisible(Sys.sleep(10))\n" nil (get-process ess-local-process-name) ;; test only
-     (lambda (proc) (process-put proc 'packages-cached? t))))
+  ;; (when ess-can-eval-in-background
+  ;;   (ess-async-command-delayed
+  ;;    "invisible(installed.packages())\n" nil (get-process ess-local-process-name)
+  ;;    ;; "invisible(Sys.sleep(10))\n" nil (get-process ess-local-process-name) ;; test only
+  ;;    (lambda (proc) (process-put proc 'packages-cached? t))))
 
+  ;; sometimes needed (MM w/ Emacs 25.1, on F24 where PAGER is 'more'):
+  ;; carefully set "pager" option  "when needed":
+  (ess-eval-linewise
+   (format
+    "if(identical(getOption('pager'), file.path(R.home('bin'), 'pager'))) # rather take the ESS one
+      options(pager='%s')\n" inferior-ess-pager)
+   ;; Even more careful / sophisticated :
+   ;;  "if(identical(getOption('pager'), file.path(R.home('bin'), 'pager')) &&
+   ;;  grepl('\\<more\\>', .P <- Sys.getenv('PAGER'))) {  # rather take the ESS one
+   ;;    cat('$PAGER has more: ', sQuote(.P), '\\n --> setting R`s pager option():\\n')
+   ;;    options(pager='%s')\n}\n" inferior-ess-pager)
+   nil nil nil 'wait)
   (inferior-ess-r-load-ESSR)
 
   (when inferior-ess-language-start
@@ -1034,9 +1103,9 @@ similar to `load-library' emacs function."
     (concat cmd "('" file "'" args ")\n")))
 
 (defun ess-r-build-eval-message (message)
-  (let ((env (cond ((ess-r-get-evaluation-env))
-                   (ess-debug-minor-mode
-                    (substring-no-properties ess-debug-indicator 1)))))
+  (let ((env (cond (ess-debug-minor-mode
+                    (substring-no-properties ess-debug-indicator 1))
+                   ((ess-r-get-evaluation-env)))))
     (if env
         (format "[%s] %s" env message)
       message)))
@@ -1083,7 +1152,7 @@ attached packages."
       (message (format "Evaluating in %s" (propertize env 'face font-lock-function-name-face))))
     (force-mode-line-update)))
 
-(defvar-local ess-r--evaluation-env-mode-line 
+(defvar-local ess-r--evaluation-env-mode-line
   '(:eval (let ((env (ess-r-get-evaluation-env)))
             (if env
                 (format " %s"
@@ -1311,23 +1380,21 @@ we flush the cache.")
   :type 'hook
   :group 'ess-R)
 
-(defun inferior-ess-r-reload (&optional start-args)
-  "Reload R and the currently activated developer package, if any."
-  (interactive)
-  (ess-force-buffer-current)
+(ess-defmethod R inferior-ess-reload (&optional start-args)
+  (inferior-ess-r-force)
   (let ((pkg-info ess-r-package-info)
         (r-proc (ess-get-process)))
     (with-ess-process-buffer nil
       (ess-quit 'no-save)
       (while (memq (process-status r-proc) '(run busy))
         (accept-process-output r-proc 0.002))
-      (kill-buffer)
       (R start-args)
-      (when pkg-info
-        (setq-local ess-r-package-info pkg-info)
-        (ess-r-devtools-load-package))
       (run-hooks 'inferior-ess-r-reload-hook))))
 
+(defun inferior-ess-r-force (&optional prompt force no-autostart ask-if-1)
+  (setq ess-dialect "R")
+  (ess-force-buffer-current prompt force no-autostart ask-if-1))
+
 
 ;;*;; Editing Tools
 
@@ -1372,7 +1439,7 @@ Return the amount the indentation changed by."
 
 (defun ess-indent-call (&optional start)
   (save-excursion
-    (when (ess-climb-outside-calls)
+    (when (ess-escape-calls)
       (setq start (or start (point)))
       (skip-chars-forward "^[(")
       (forward-char)
@@ -1420,7 +1487,7 @@ Returns nil if line starts inside a string, t if in a comment."
       (ess-back-to-indentation)
       (cond
        ;; Strings
-       ((ess-point-in-string-p state)
+       ((ess-within-string-p state)
         (current-indentation))
        ;; Comments
        ((ess-calculate-indent--comments))
@@ -1470,7 +1537,7 @@ Returns nil if line starts inside a string, t if in a comment."
       comment-column))))
 
 (defun ess-calculate-indent--comma ()
-  (when (ess-point-in-call-p)
+  (when (ess-within-call-p)
     (let ((indent (save-excursion
                     (ess-calculate-indent--args)))
           (unindent (progn (skip-chars-forward " \t")
@@ -1484,9 +1551,9 @@ Returns nil if line starts inside a string, t if in a comment."
            (eq (char-before) ?,))
          (ess-calculate-indent--args nil))
         ((save-excursion
-           (and (ess-climb-operator)
-                (or (not ess-align-continuations-in-calls)
-                    (ess-looking-at-definition-op-p))))
+           (and (ess-ahead-operator-p)
+                (or (ess-ahead-definition-op-p)
+                    (not ess-align-continuations-in-calls))))
          (ess-calculate-indent--continued))
         (t
          (ess-calculate-indent--args 0))))
@@ -1496,7 +1563,7 @@ Returns nil if line starts inside a string, t if in a comment."
    ;; Block is an argument in a function call
    ((when containing-sexp
       (ess-at-containing-sexp
-        (ess-looking-at-call-opening "[[(]")))
+        (ess-behind-call-opening-p "[[(]")))
     (ess-calculate-indent--block 0))
    ;; Top-level block
    ((null containing-sexp) 0)
@@ -1526,7 +1593,7 @@ Returns nil if line starts inside a string, t if in a comment."
                     (containing-sexp
                      (when (ess-at-containing-sexp
                              (looking-at "{"))
-                       (ess-climb-outside-prefixed-block))))
+                       (ess-escape-prefixed-block))))
                    (some 'looking-at (ess-overridden-blocks)))
           (+ (current-column) offset))))))
 
@@ -1548,7 +1615,7 @@ Returns nil if line starts inside a string, t if in a comment."
         (+ (current-column)
            (ess-offset 'block)))
        ;; Don't indent relatively other continuations
-       ((ess-looking-at-continuation-p)
+       ((ess-ahead-continuation-p)
         nil)
        ;; If a block already contains an indented line, we can indent
        ;; relatively from that first line
@@ -1590,19 +1657,19 @@ Returns nil if line starts inside a string, t if in a comment."
      ((ess-at-indent-point
         (and (ess-unbraced-block-p)
              (goto-char containing-sexp)
-             (ess-looking-at-call-opening "[[(]")))
+             (ess-behind-call-opening-p "[[(]")))
       'body)
      ;; Indentation of opening brace as argument
      ((ess-at-containing-sexp
-        (ess-looking-at-call-opening "[[(]"))
+        (ess-behind-call-opening-p "[[(]"))
       'opening)
      ;; Indentation of body or closing brace as argument
      ((ess-at-containing-sexp
         (and (or (looking-at "{")
-                 (ess-looking-at-block-paren-p))
+                 (ess-behind-block-paren-p))
              prev-containing-sexp
              (goto-char prev-containing-sexp)
-             (ess-looking-at-call-opening "[[(]")))
+             (ess-behind-call-opening-p "[[(]")))
       'body))))
 
 (defun ess-calculate-indent--block (&optional offset)
@@ -1631,7 +1698,7 @@ Returns nil if line starts inside a string, t if in a comment."
                               (ess-unbraced-block-p))
                             'unbraced)
                            ((ess-at-containing-sexp
-                              (not (ess-looking-back-attached-name-p)))
+                              (not (ess-ahead-attached-name-p)))
                             'bare-block)
                            (t)))
          (call-pos (if (and (not (eq block-type 'unbraced))
@@ -1798,7 +1865,7 @@ Returns nil if line starts inside a string, t if in a comment."
 ;; name or its closing delim)
 (defun ess-move-to-leftmost-side ()
   (when (or (looking-at "[({]")
-            (ess-looking-at-call-p))
+            (ess-behind-call-p))
     (ess-save-excursion-when-nil
       (let ((start-col (current-column)))
         (skip-chars-forward "^{[(")
@@ -1826,7 +1893,7 @@ Returns nil if line starts inside a string, t if in a comment."
                  (and (memq 'fun-decl-opening ess-indent-from-lhs)
                       (string= block-type "function")
                       (ess-climb-operator)
-                      (ess-looking-at-assignment-op-p)
+                      (ess-behind-assignment-op-p)
                       (ess-climb-expression)))
                (current-column))
               ((= (save-excursion
@@ -1863,7 +1930,7 @@ otherwise nil."
          (t
           (let ((first-indent (or (eq climbed 'def-op)
                                   (save-excursion
-                                    (when (ess-looking-back-closing-p)
+                                    (when (ess-ahead-closing-p)
                                       (ess-climb-expression))
                                     (not (ess-climb-continuations cascade))))))
             ;; Record all indentation levels between indent-point and
@@ -1874,7 +1941,7 @@ otherwise nil."
             ;; Indenting continuations from the front of closing
             ;; delimiters looks better
             (when
-                (ess-looking-back-closing-p)
+                (ess-ahead-closing-p)
               (backward-char))
             (+ (min (current-column) max-col)
                (cond
@@ -1923,8 +1990,8 @@ otherwise nil."
                (goto-char (1+ contained-sexp))
                (ess-up-list))
               ;; Jump over continued statements
-              ((and jump-cont (ess-looking-back-operator-p))
-               (ess-skip-blanks-forward t)
+              ((and jump-cont (ess-ahead-operator-p 'strict))
+               (ess-climb-token)
                (ess-jump-continuations))
               ;; Jump over comments
               ((looking-at "#")
@@ -2041,71 +2108,77 @@ otherwise nil."
                         (ess-jump-char ","))
               (setq i (1- i))))
           (newline-and-indent)))
-        (while (and (not (looking-at "[])]"))
-                    (/= (point) (or last-pos 1))
-                    (not infinite))
-          (setq prefix-break nil)
-          ;; Record start-pos as future breaking point to avoid breaking
-          ;; at `=' sign
-          (while (looking-at "[ \t]*[\n#]")
-            (forward-line)
-            (ess-back-to-indentation))
-          (setq start-pos (point))
-          (while (and (< (current-column) fill-column)
-                      (not (looking-at "[])]"))
-                      (/= (point) (or last-pos 1))
-                      ;; Break after one pass if prefix is active
-                      (not prefix-break))
-            (when (memq style '(2 3))
-              (setq prefix-break t))
-            (ess-jump-char ",")
-            (setq last-pos (point))
-            ;; Jump expression and any continuations. Reindent all lines
-            ;; that were jumped over
-            (let ((cur-line (line-number-at-pos))
-                  end-line)
-              (when (ess-jump-arg)
-                (setq last-newline nil))
-              (save-excursion
-                (when (< cur-line (line-number-at-pos))
-                  (setq end-line (line-number-at-pos))
-                  (ess-goto-line (1+ cur-line))
-                  (while (and (<= (line-number-at-pos) end-line)
-                              (/= (point) (point-max)))
-                    (ess-indent-line)
-                    (forward-line))))))
-          (when (or (>= (current-column) fill-column)
-                    prefix-break
-                    ;; Ensures closing delim on a newline
-                    (and (= style 4)
-                         (looking-at "[ \t]*[])]")
-                         (setq last-pos (point))))
-            (if (and last-pos (/= last-pos start-pos))
-                (goto-char last-pos)
-              (ess-jump-char ","))
-            (cond ((looking-at "[ \t]*[#\n]")
-                   (forward-line)
-                   (ess-indent-line)
-                   (setq last-newline nil))
-                  ;; With levels 2 and 3, closing delim goes on a newline
-                  ((looking-at "[ \t]*[])]")
-                   (when (and (memq style '(2 3 4))
-                              ess-fill-calls-newlines
-                              (not last-newline))
-                     (newline-and-indent)
-                     ;; Prevent indenting infinitely
-                     (setq last-newline t)))
-                  ((not last-newline)
-                   (newline-and-indent)
-                   (setq last-newline t))
-                  (t
-                   (setq infinite t)))))
+        (ess-fill-args--roll-lines)
         ;; Reindent surrounding context
         (ess-indent-call (car bounds)))
       ;; Signal marker for garbage collection
       (set-marker (cadr bounds) nil)
       (undo-boundary))))
 
+(defun ess-fill-args--roll-lines ()
+  (while (and (not (looking-at "[])]"))
+              (/= (point) (or last-pos 1))
+              (not infinite))
+    (setq prefix-break nil)
+    ;; Record start-pos as future breaking point to avoid breaking
+    ;; at `=' sign
+    (while (looking-at "[ \t]*[\n#]")
+      (forward-line)
+      (ess-back-to-indentation))
+    (setq start-pos (point))
+    (while (and (< (current-column) fill-column)
+                (not (looking-at "[])]"))
+                (/= (point) (or last-pos 1))
+                ;; Break after one pass if prefix is active
+                (not prefix-break))
+      (when (memq style '(2 3))
+        (setq prefix-break t))
+      (ess-jump-token ",")
+      (setq last-pos (point))
+      ;; Jump expression and any continuations. Reindent all lines
+      ;; that were jumped over
+      (let ((cur-line (line-number-at-pos))
+            end-line)
+        (cond ((ess-jump-arg)
+               (setq last-newline nil))
+              ((ess-token-after= ",")
+               (setq last-newline nil)
+               (setq last-pos (1- (point)))))
+        (save-excursion
+          (when (< cur-line (line-number-at-pos))
+            (setq end-line (line-number-at-pos))
+            (ess-goto-line (1+ cur-line))
+            (while (and (<= (line-number-at-pos) end-line)
+                        (/= (point) (point-max)))
+              (ess-indent-line)
+              (forward-line))))))
+    (when (or (>= (current-column) fill-column)
+              prefix-break
+              ;; Ensures closing delim on a newline
+              (and (= style 4)
+                   (looking-at "[ \t]*[])]")
+                   (setq last-pos (point))))
+      (if (and last-pos (/= last-pos start-pos))
+          (goto-char last-pos)
+        (ess-jump-char ","))
+      (cond ((looking-at "[ \t]*[#\n]")
+             (forward-line)
+             (ess-indent-line)
+             (setq last-newline nil))
+            ;; With levels 2 and 3, closing delim goes on a newline
+            ((looking-at "[ \t]*[])]")
+             (when (and (memq style '(2 3 4))
+                        ess-fill-calls-newlines
+                        (not last-newline))
+               (newline-and-indent)
+               ;; Prevent indenting infinitely
+               (setq last-newline t)))
+            ((not last-newline)
+             (newline-and-indent)
+             (setq last-newline t))
+            (t
+             (setq infinite t))))))
+
 (defun ess-fill-continuations (&optional style)
   (let ((bounds (ess-continuations-bounds 'marker))
         (undo-inhibit-record-point t)
