@@ -35,6 +35,7 @@
 (autoload 'inferior-ess "ess-inf" "Run an ESS process.")
 (autoload 'ess-mode     "ess-mode" "Edit an ESS process.")
 
+(require 'ess-utils)
 (require 'ess-sta-l)
 
 (defvar STA-dialect-name "stata"
@@ -53,6 +54,7 @@
     (ess-loop-timeout              . 500000 )
     (ess-object-name-db-file       . "ess-sta-namedb.el" )
     (ess-help-web-search-command   . "http://www.stata.com/search/?q=%s&restrict=&btnG=Search&client=stata&num=&output=xml_no_dtd&site=stata&ie=&oe=UTF-8&sort=&proxystylesheet=stata")
+    (ess-eval-linewise-function    . #'stata-eval-linewise)
     (inferior-ess-font-lock-defaults . ess-STA-mode-font-lock-defaults)
     (inferior-ess-program          . inferior-STA-program-name)
     (inferior-ess-objects-command  . "describe\n")
@@ -70,6 +72,7 @@
     (comment-end                  . " \*/")
     (comment-start-skip           . "/\\*+ *")
     (comment-use-syntax           . t) ;; needed for multiline
+    (ess-execute-screen-options-command . "set linesize %s\n")
     )
   "Variables to customize for Stata.")
 
@@ -125,9 +128,7 @@ This function is placed in `ess-presend-filter-functions'.
       (goto-char (point-max))
       (with-current-buffer (process-buffer proc)
         (add-hook 'ess-presend-filter-functions 'ess-sta-remove-comments nil 'local)
-        (run-mode-hooks 'ess-stata-post-run-hook))
-      )
-    ))
+        (run-mode-hooks 'ess-stata-post-run-hook)))))
 
 
 (defun STA-transcript-mode ()
@@ -151,8 +152,16 @@ This function is placed in `ess-presend-filter-functions'.
   (or (ess-process-get 'help-topics)
       (progn
         (ess-process-put 'help-topics (ess--STA-retrive-topics-from-search))
-        (ess-process-get 'help-topics))
-      ))
+        (ess-process-get 'help-topics))))
+
+(defun stata-eval-linewise (text &optional invisibly &rest args)
+  ;; The following is required to make sure things work!
+  (let ((ess-eval-linewise-function nil)
+        ;; RAS: mindless replacement of semi-colons
+        (text (if ess-sta-delimiter-friendly
+                  (ess-replace-in-string text ";" "\n")
+                text)))
+    (apply #'ess-eval-linewise text t args)))
 
  ; Provide package
 
