@@ -100,6 +100,22 @@
 ;;;=====================================================
 ;;;
 
+(defun ess-write-to-dribble-buffer (text)
+  "Write TEXT to dribble ('*ESS*') buffer."
+  (unless (buffer-live-p ess-dribble-buffer)
+    ;; ESS dribble buffer must be re-created.
+    (setq ess-dribble-buffer (get-buffer-create "*ESS*")))
+  (let (deactivate-mark)
+    (with-current-buffer ess-dribble-buffer
+      (goto-char (point-max))
+      (insert-before-markers text))))
+
+;; Shortcut to render "dribbling" statements less cluttering:
+(defun ess-if-verbose-write (text)
+  "Write TEXT to dribble buffer ('*ESS*') only *if* `ess-verbose'."
+  (if ess-verbose (ess-write-to-dribble-buffer text)))
+
+
 (require 'easymenu)
 (if (or window-system
         noninteractive ; compilation!
@@ -416,8 +432,7 @@ Otherwise try a list of fixed known viewers.
                     (executable-find "acroread")
                     (executable-find "xdg-open")
                     ;; this one is wrong, (ok for time being as it is used only in swv)
-                    (car (ess-get-words-from-vector
-                          "getOption(\"pdfviewer\")\n"))
+                    (car (ess-get-words-from-vector "getOption(\"pdfviewer\")\n"))
                     )))
     (when (stringp viewer)
       (setq viewer (file-name-nondirectory viewer)))
@@ -445,21 +460,6 @@ Otherwise try a list of fixed known viewers.
       (store-match-data (match-data))
       (nreverse list))))
 
-(defun ess-write-to-dribble-buffer (text)
-  "Write TEXT to dribble ('*ESS*') buffer."
-  (unless (buffer-live-p ess-dribble-buffer)
-    ;; ESS dribble buffer must be re-created.
-    (setq ess-dribble-buffer (get-buffer-create "*ESS*")))
-  (let (deactivate-mark)
-    (with-current-buffer ess-dribble-buffer
-      (goto-char (point-max))
-      (insert-before-markers text))))
-
-;; Shortcut to render "dribbling" statements less cluttering:
-(defun ess-if-verbose-write (text)
-  "Write TEXT to dribble buffer ('*ESS*') only *if* `ess-verbose'."
-  (if ess-verbose (ess-write-to-dribble-buffer text)))
-
 
 (defvar ess--make-local-vars-permenent nil
   "If this varialbe is non-nil in a buffer make all variable permannet.
@@ -517,7 +517,7 @@ Used in noweb modes.")
 
 ;; Toby Speight <Toby.Speight@ansa.co.uk>
 ;;> ;; untested
-;;> (let ((l R-customize-alist))            ; or whatever
+;;> (let ((l ess-r-customize-alist))            ; or whatever
 ;;>   (while l
 ;;>     (set (car (car l)) (cdr (car l)))   ; set, not setq!
 ;;>     (setq l (cdr l))))
@@ -533,18 +533,18 @@ Used in noweb modes.")
 ;; Erik Naggum <erik@naggum.no>
 ;;
 ;;(mapcar (lambda (pair) (set (car pair) (cdr pair)))
-;;        R-customize-alist)
+;;        ess-r-customize-alist)
 ;;
 ;;if you want to evaluate these things along the way, which it appears that
 ;;you want, try:
 ;;
 ;;(mapcar (lambda (pair) (set (car pair) (eval (cdr pair))))
-;;        R-customize-alist)
+;;        ess-r-customize-alist)
 
 ;; jsa@alexandria.organon.com (Jon S Anthony)
 ;;(mapcar (lambda (x)
 ;;          (set-variable (car x) (cdr x)))
-;;      R-customize-alist)
+;;      ess-r-customize-alist)
 
 
 
