@@ -106,14 +106,14 @@ return new alist whose car is the new pair and cdr is ALIST.
          (dialect (or dialect
                       (ess-completing-read "Dialect" dialects nil t))))
     (cond
-     ((string= dialect "julia") julia-customize-alist)
+     ((string= dialect "julia") ess-julia-customize-alist)
      ((string= dialect "arc")   ARC-customize-alist)
      ((string= dialect "vst")   VST-customize-alist)
      ((string= dialect "omg")   OMG-customize-alist)
      ((string= dialect "s3")    S3-customize-alist)
      ((string= dialect "s4")    S4-customize-alist)
      ((string= dialect "stata") STA-customize-alist)
-     ((string= dialect "R")     R-customize-alist )
+     ((string= dialect "R")     ess-r-customize-alist)
      ((string= dialect "sp3")   S+3-customize-alist)
      ((string= dialect "sp4")   S+4-customize-alist)
      ((string= dialect "sqpe4") Sqpe+4-customize-alist)
@@ -199,7 +199,7 @@ DIALECT is the desired ess-dialect. If nil, ask for dialect"
       ;; ugly fix for evn variable. What can we do :(
       (ess-eval-linewise (format "options(pager='%s')\n" inferior-ess-pager)
                          nil nil nil 'wait)
-      (ess--R-load-ESSR))
+      (inferior-ess-r-load-ESSR))
 
     (when (equal ess-dialect "S+")
       (ess-command ess-S+--injected-code))
